<#
.SYNOPSIS
    A script that demonstrates PowerShell integration with BWS REST API.
.DESCRIPTION
    Authenticate and call either "Create local user", "Delete user" or "Update local user" BWS REST API depending what actionType is used.
.PARAMETER uemHostAndPort
    Host and port of the UEM.
.PARAMETER tenantGuid
    TenantGuid to use for authentication.
.PARAMETER authType
    Authentication type to use.  (Local or AD).
.PARAMETER username
    Username to use for authentication.
.PARAMETER password
    Password to use for authentication.
.PARAMETER domain
    Domain to use for authentication if authType=AD.
.PARAMETER actionType
    Action to perform. (CreateUser, DeleteUser or UpdateUserPassword)
.PARAMETER emailAddress
    Email address of the user to perform the action on.  Manditory for DeleteUser and UpdateUserPassword.
.PARAMETER newUsername
    New username for CreateUser.
.PARAMETER newDisplayName
    New display name for CreateUser.
.PARAMETER newFirstName
    New first name for CreateUser.
.PARAMETER newLastName
    New last name for CreateUser.
.PARAMETER newEmailAddress
    New email address for CreateUser.
.PARAMETER newUserPassword
    New user password for CreateUser or UpdateUserPassword.
.LINK
    https:// help.blackberry.com/en/blackberry-web-services-for-blackberry-uem/current/
.EXAMPLE
    .\CreateDeleteUpdateUserPassword.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -actionType CreateUser -newUsername pmorley -newDisplayName pmorley -newFirstName Paul -newLastName Morley -newEmailAddress pmorley@example.com -newUserPassword password123
    .\CreateDeleteUpdateUserPassword.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType Local -username admin -password password -actionType DeleteUser -emailAddress pmorley@example.com
    .\CreateDeleteUpdateUserPassword.ps1 -uemHostAndPort server01:18084 -tenantGuid SRP00000 -authType AD -username admin -password password -domain example -actionType UpdateUserPassword -emailAddress pmorley@example.com -newUserPassword password123
#>

Param(
    [Parameter(
        Mandatory=$true,
        HelpMessage="UEM Host and port."
    )]
    [ValidateNotNullorEmpty()]
    [string]$uemHostAndPort,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Tenant guid to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$tenantGuid,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (AD | Local) authType."
    )]
    [ValidateSet('AD', 'Local')]
    [string]$authType,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Username to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$username,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Password to authenticate with."
    )]
    [ValidateNotNullorEmpty()]
    [string]$password,

    [Parameter(
        Mandatory=$false
    )]
    [string]$domain,

    [Parameter(
        Mandatory=$true,
        HelpMessage="Choose (CreateUser | DeleteUser | UpdateUserPassword) actionType."
    )]
    [ValidateSet('CreateUser', 'DeleteUser', 'UpdateUserPassword')]
    [string]$actionType,

    [Parameter(
        Mandatory=$false
    )]
    [string]$emailAddress,

    [Parameter(
        Mandatory=$false
    )]
    [string]$newUsername,

    [Parameter(
        Mandatory=$false
    )]
    [string]$newDisplayName,

    [Parameter(
        Mandatory=$false
    )]
    [string]$newFirstName,

    [Parameter(
        Mandatory=$false
    )]
    [string]$newLastName,

    [Parameter(
        Mandatory=$false
    )]
    [string]$newEmailAddress,

    [Parameter(
        Mandatory=$false
    )]
    [string]$newUserPassword
)

$global:authString = $null;
$global:uemBaseUrl = $null;
[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

<#
    * Generate authorization header required to perform the REST call.
    *
    * @return
    *       True if successful, false otherwise.
#>
function setup()
{
    $methodName = "setup()";
    enterMethod($methodName);
    $provider = $null;

    $global:uemBaseUrl = "https://" + $uemHostAndPort + "/" + $tenantGuid + "/api/v1";

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Content-Type", "application/vnd.blackberry.authorizationrequest-v1+json")

    # Create body for REST call
    $body = @{}
    $body.add("provider",$authType);
    $body.add("username", $username);
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));
    if($authType -eq "AD")
    {
        if(-not($domain))
        {
            throw [System.ArgumentNullException]"The domain argument is required when authType=AD";
        }
        $body.add("domain", $domain);
    }

    $request = $global:uemBaseUrl + "/util/authorization";

    # Invoke RESTMethod to call authorization route to generate the authorization headers
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
        $global:authString = $response
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("Authorization header string generated: {0}", $global:authString));
    Write-Host("Setup Complete...");
    exitMethod($methodName);
    return $true;
}

<#
    * Generate authorization header required to perform the REST call.
    *
    * @param email
    *       Email of the user to get.
    * @return
    *       User that was found.
#>
function getUser($email)
 {
    $methodName = "getUser()";
    enterMethod($methodName);
    $response = $null;

    # Create and set the headers for the REST request
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:uemBaseUrl + "/users?query=emailAddress=" + $email;

    # Invoke RESTMethod to call users API to obtain users in the system
    try
    {
        $response = Invoke-RestMethod -Method GET -Headers $headers -Uri $request;
        if (($response.users -ne $null) -and ($response.users.length -eq 1))
        {
            Write-Host([string]::Format("User with email address '{0}' was found", $email));
        }
        else
        {
            Write-Host([string]::Format("User with email address '{0}' not found", $email));
            exit(1);
        }
    }
    catch
    {
         handleError $_;
    }
    exitMethod($methodName);
    return $response;
 }

<#
    * Add a local user.
    *
    * @param newUsername
    * @param newDisplayName
    * @param newFirstName
    * @param newLastName
    * @param newEmailAddress
    * @param newUserPassword
#>
function createLocalUser($newUsername, $newDisplayName, $newFirstName, $newLastName, $newEmailAddress, $newUserPassword)
{
    $methodName = "createLocalUser()";
    enterMethod($methodName);

    if([string]::IsNullOrEmpty($newUsername))
    {
        throw [System.ArgumentNullException]"The newUsername argument is required when actionType=CreateUser";
    }
    if([string]::IsNullOrEmpty($newDisplayName))
    {
        throw [System.ArgumentNullException]"The newDisplayName argument is required when actionType=CreateUser";
    }

    # Create and set the headers for the REST command
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.user-v1+json");

    # Create body for REST call
    $body = @{};
    $body.add("username",$newUsername);
    $body.add("displayName", $newDisplayName);
    if (-not ([string]::IsNullOrEmpty($newFirstName)))
    {
        $body.add("firstName", $newFirstName);
    }
    if (-not ([string]::IsNullOrEmpty($newLastName)))
    {
        $body.add("lastName", $newLastName);
    }
    if (-not ([string]::IsNullOrEmpty($newEmailAddress)))
    {
        $body.add("emailAddress", $newEmailAddress);
    }
    if (-not ([string]::IsNullOrEmpty($newUserPassword)))
    {
        $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($newUserPassword)));
    }
    $request = $global:uemBaseUrl + "/users";

    # Invoke RESTMethod to create a local user.
    try
    {
        $response = Invoke-RestMethod -Method POST -Headers $headers -Uri $request -Body ($body | ConvertTo-Json);
    }
    catch
    {
        handleError $_;
    }

    Write-Host([string]::Format("User {0} with guid {1} was created successfully", $response.displayName, $response.guid));
    exitMethod($methodName);
}

<#
    * Delete user.
    *
    * @param user
    *       User to delete.
#>
function deleteUser($user)
{
    $methodName = "deleteUser()";
    enterMethod($methodName);

    # Create and set the headers for the REST command
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.users-v1+json");

    $request = $global:uemBaseUrl + "/users/" + $user.users.guid;

    # Invoke RESTMethod to remove UEM user.
    try
    {
        $response = Invoke-RestMethod -Method DELETE -Headers $headers -Uri $request;
    }
    catch
    {
        handleError $_;
    }
    Write-Host([string]::Format("User {0} was removed successfully", $user.users.emailAddress));
    exitMethod($methodName);
}

<#
    * Update user password.
    *
    * @param user
    *       User to delete.
    * @param password
    *       New password.
#>
function upatePassword($user, $password)
{
    $methodName = "upatePassword()";
    enterMethod($methodName);

    # Create and set the headers for the REST command
    $headers = @{}
    $headers.add("Authorization", $global:authString);
    $headers.add("Content-Type", "application/vnd.blackberry.user-v1+json");

    $body = @{};
    $body.add("password", [Convert]::ToBase64String([System.Text.Encoding]::UTF8.GetBytes($password)));

    $request = $global:uemBaseUrl + "/users/" + $user.users.guid;

    # Invoke RESTMethod to update local UEM users password
    try
    {
        $response = Invoke-RestMethod -Method PATCH -Headers $headers -Uri $request  -Body ($body | ConvertTo-Json);
    }
    catch
    {
        handleError $_;
    }
    Write-Host([string]::Format("Password was updated for {0}", $user.users.emailAddress));
    exitMethod($methodName);
}

<#
    * Logging for method entry.
    *
    * @param methodName
    *       Name of the method to log
#>
function enterMethod($methodName)
{
    Write-Host([string]::Format("Entering {0}...", $methodName));
}

<#
    * Logging for method exit.
    *
    * @param methodName
    *       Name of the method to log.
#>
function exitMethod($methodName)
{
    Write-Host([string]::Format("Exiting {0}...", $methodName));
}

<#
    * Handle any errors that occur when performing the REST call.
    *
    * @param error
    *       The group create context.
#>
function handleError($error)
{
    Write-Error ("The command cound not be completed.  `r`nReason: " + $error.exception.message + " `r`nDetails: "+ $error.errordetails);
    exit(1);
}
function main
{
    $methodName = "main()";
    enterMethod($methodName);
    try
    {
        if (setup)
        {
            if($actionType -eq "CreateUser")
            {
                createLocalUser $newUsername $newDisplayName $newFirstName $newLastName $newEmailAddress $newUserPassword;
            }
            elseif($actionType -eq "DeleteUser")
            {
                $getUserResponse = getUser $emailAddress;
                deleteUser $getUserResponse;
            }
            elseif($actionType -eq "UpdateUserPassword")
            {
                if([string]::IsNullOrEmpty($newUserPassword))
                {
                    throw [System.ArgumentNullException]"The newUserPassword argument is required when actionType=UpdateUserPassword";
                }

                $getUserResponse = getUser $emailAddress;
                if([string]::IsNullOrWhiteSpace($getUserResponse.users.directoryId))
                {
                    upatePassword $getUserResponse $newUserPassword;
                }
                else
                {
                    Write-Host([string]::Format("User with email '{0}' is a directory user. Only local users are supported.", $emailAddress));
                }
            }
        }
    }
    catch
    {
        handleError $_;
    }
    exitMethod($methodName);
    Write-Host("Complete!");
    exit(0);
}

# Call main function.
main;